<?php
/**
 * Pocket Project
 * 
 * @copyright 2006 Power-Station co., Ltd, ALL Rights Reserved,
 */

require_once 'XML/RPC.php';

/**
 *  Pingクラス
 *
 *  @author     Koji Hasegawa
 *  @access     public
 *  @package    Pocket
 */
class Pocket_Ping
{
    /** @var    string  _site_name          サイト名称 */
    var $_site_name;

    /** @var    string  _site_url           サイトURL */
    var $_site_url;

    /** @var    string  _rss_url            RSSURL */
    var $_rss_url;

    /** @var    string  _timeout            タイムアウト値 */
    var $_timeout;

    /** @var    string  _ping_url           PING送信先URL */
    var $_ping_url;

    /** @var    string  _ping_url           PING送信モード */
    var $_ping_mode;

    /** @var    string  _err_code           エラーコード */
    var $_err_code;

    /** @var    string  _err_msg            エラーメッセージ */
    var $_err_msg;

    /**
     *  コンストラクタ
     *
     *  @access public
     */
    function Pocket_Ping()
    {
    }

    /**
     *  PING送信
     *
     *  @access public
     *  @param  string  $site_name          サイト名称
     *  @param  string  $site_url           サイトURL
     *  @param  string  $rss_url            RSSURL
     *  @param  string  $timeout            タイムアウト値
     *  @param  string  $ping_url           PING送信先URL
     *  @param  string  $ping_mode          PING送信モード
     */
    function send($site_name, $site_url, $rss_url, $timeout, $ping_url, $ping_mode)
    {
        // 初期値設定
        $this->_site_name = $site_name;
        $this->_site_url  = $site_url;
        $this->_rss_url   = $rss_url;
        $this->_timeout   = $timeout;
        $this->_ping_url  = $ping_url;
        $this->_ping_mode = $ping_mode;
        $this->_err_code  = '';
        $this->_err_msg   = '';

        // 送信先URLのパース
        $ping_server = $this->parse_pingurl($this->_ping_url);

        // RPC_Clientの生成
        $client = new XML_RPC_Client($ping_server['path'], $ping_server['host']);

        // モード別にメッセージを組み立てを変更する
        switch ($this->_ping_mode)
        {
            case '1':
                // 通常モード
                $msg = $this->normal_ping();
                break;
            case '2':
                // RSS付モード
                $msg = $this->rssplus_ping();
                break;
            case '3':
                // 拡張モード
                $msg = $this->extend_ping();
                break;
            default:
                // 値がない場合は通常モード
                $msg = $this->normal_ping();
                break;
        }

        // メッセージ送信
        $response = $client->send($msg, $timeout);

        if ($response)
        {
            // サーバーから応答があった場合
            if (!$response->faultCode())
            {
                // メッセージを取得
                $message = $response->value(); 
                // メッセージ内の配列からflerrorとmessageを取得する
                $val_array = $message->getval();
                if ($val_array['flerror'])
                {
                    // 0以外(true)の場合エラー
                    $this->_err_code = $val_array['flerror'];
                    $this->_err_msg = $val_array['message'];
                    return false;
                }
            }
            else {
                // サーバーから応答はあったがエラーであった場合
                $this->_err_code = $response->faultCode();
                $this->_err_msg = $response->faultString();
                return false;
            }
        }
        else {
           // サーバーから応答がなかった場合
           $this->_err_code = $client->errno;
           $this->_err_msg = $client->errstr;
           return false;
        }

        return true;
    }

    /**
     *  PINGサーバーURLのパース
     *
     *  @access public
     */
    function parse_pingurl($url)
    {
        $parse = parse_url($url);

        if ($parse['query'])
        {
            // クエリがある場合は連結して返却
            $parse['path'] = $parse['path'] . "?" .$parse['query'];
        }
        // http:// | https:// の形にして返却
        $parse['host'] = $parse['scheme'] . "://" . $parse['host'];

        return $parse;
    }

    /**
     *  PING送信（通常）
     *  weblogUpdates.ping (weblogname, weblogurl)
     *
     *  @access public
     */
    function normal_ping()
    {
        $msg = new XML_RPC_Message("weblogUpdates.ping");
        $msg->addParam(new XML_RPC_Value($this->_site_name));
        $msg->addParam(new XML_RPC_Value($this->_site_url));

        return $msg;
    }

    /**
     *  PING送信（RSS付）
     *  weblogUpdates.ping (weblogname, weblogurl, siteurl, rssurl)
     *
     *  @access public
     */
    function rssplus_ping()
    {
        $msg = new XML_RPC_Message("weblogUpdates.ping");
        $msg->addParam(new XML_RPC_Value($this->_site_name));
        $msg->addParam(new XML_RPC_Value($this->_site_url));
        $msg->addParam(new XML_RPC_Value($this->_site_url));
        $msg->addParam(new XML_RPC_Value($this->_rss_url));

        return $msg;
    }

    /**
     *  PING送信（拡張）
     *  weblogUpdates.extendedPing (weblogname, weblogurl, contenturl, rssurl)
     *
     *  @access public
     */
    function extend_ping()
    {
        $msg = new XML_RPC_Message("weblogUpdates.extendedPing");
        $msg->addParam(new XML_RPC_Value($this->_site_name));
        $msg->addParam(new XML_RPC_Value($this->_site_url));
        $msg->addParam(new XML_RPC_Value($this->_site_url));
        $msg->addParam(new XML_RPC_Value($this->_rss_url));

        return $msg;
    }
}

?>
